/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.bsvrz.dua.bastband.bastband.dvs.funclib.logger;

import java.lang.reflect.Field;

import de.bsvrz.sys.funclib.debug.Debug;

/**
 *
 * <p>
 * LoggerFactory f&uuml;r Applikationen mit Datenverteiler.
 * </p>
 *
 * <p>
 * Das Log-System basiert auf <code>java.util.logging</code>. Ob ein Log-Befehl
 * wirklich auf ein Channel ausgegeben wird, ist vom dem verwendeten Log-Objekt
 * zugordneten LogLevel abh&auml;ngig und davon, ob es einen LogChannel gibt,
 * der den entsprechenen LogLevel ausgibt.
 * </p>
 *
 * <p style="margin:0">
 * Durch die Datenverteiler-Funktionen werden folgene Parameter interpretiert.
 *
 * <pre>
 * -debugFileSize=10000000
 * -debugFileCount=5
 * -debugLevelStdErrText=WARNING
 * -debugLevelFileText=OFF
 * -debugLevelFileXML=OFF
 * -debugLevelFileExcel=OFF
 * -debugLevelFileHTML=OFF
 * -debugFilePath=.
 * -debugGui=nein
 *
 * -debugSetLoggerAndLevel=[Bereich]:Level
 * </pre>
 *
 * Der Parameter <b>debugSetLoggerAndLevel</b> kann mehrfach angegeben werden.
 *
 * Mit den Parametern
 *
 * <pre style="margin:0">
 * -debugLevelStdErrText = ERROR|WARNING|..|FINEST|OFF
 * -debugLevelFileText   = ERROR|WARNING|..|FINEST|OFF
 * </pre>
 *
 * wird angegeben, bis zu welchem Level die Ausgabe in den LogChannel Console
 * bzw. LogChannel Log-Datei erfolgen soll.
 * <p>
 * Standardm&auml;&szlig;ig werden den Log-Objekten der Level Level.ALL
 * zugewiesen.
 * </p>
 *
 * <p>
 * Mit dem Parameter debugSetLoggerAndLevel wird der LogLevel eines LogObjektes
 * angegeben.
 *
 * <pre style="margin:0">
 * z.B.
 * -debugSetLoggerAndLevel=de:FINEST
 * -debugSetLoggerAndLevel=stauma:INFO
 * -debugSetLoggerAndLevel=puk:INFO
 * </pre>
 *
 * <p>
 * Bei der Initialisierung der Klasse Debug wird immer eine RootCategorie
 * DEFAULT-DEBUG erzeugt. Die Initialisierung mit Debug.init() führt zur
 * Erzeugung einer weiteren RootCategorie. Normalerweise werden dieser Methode
 * die Startparameter übergeben. Bei Verwendung von BaseConsoleProgram ist die
 * RootCategorie = MyBaseConsoleProgram.getClass().getSimpleName() Bei
 * Verwendung von StandardApplication ist die RootCategorie =
 * MyStandardApplication.getClass().getSimpleName()
 *
 * Mit de.logosib.dvs.funclib.logger.LoggerFactory.getRootLooger() kann die
 * RootCategorie bestimmt werden.
 * </p>
 *
 *
 * @author Ralf Trier
 * @version 1.0
 */
public class LoggerFactory extends de.bsvrz.dua.bastband.bastband.funclib.logger.LoggerFactory {

	private static boolean _NotInitialized = true;

	static String _rootName = "";

	private static synchronized void init() {
		if (LoggerFactory._NotInitialized) {
			try {
				final Field f = Debug.class.getDeclaredField("_rootName");
				f.setAccessible(true);
				LoggerFactory._rootName = f.get(null).toString();
			} catch (final Exception ex) {
			}

			final java.util.logging.Logger rootLogger = java.util.logging.Logger.getLogger(LoggerFactory._rootName);
			final java.util.logging.Handler[] handlers = rootLogger.getHandlers();
			if (handlers.length > 0) {
				handlers[0].setFormatter(new de.bsvrz.dua.bastband.bastband.funclib.logger.LogFormatter());
			}
			LoggerFactory._NotInitialized = false;
		}
	}

	public synchronized String getRootName() {
		String sRootName = "";

		try {
			final Field f = Debug.class.getDeclaredField("_rootName");
			f.setAccessible(true);
			sRootName = f.get(null).toString();
		} catch (final Exception ex) {
		}
		return sRootName;
	}

	public LoggerFactory() {
		LoggerFactory.init();
	}

	/**
	 * @return Logger
	 */
	@Override
	public de.bsvrz.dua.bastband.bastband.funclib.logger.Logger getLogger() {
		String debugClassName = "Klassenname nicht ermittelbar!";
		final Throwable t = new Throwable();
		if (t.getStackTrace().length > 1) {
			debugClassName = t.getStackTrace()[1].getClassName();
		}
		return de.bsvrz.dua.bastband.bastband.funclib.logger.StandardLogger
				.getStandardLogger(getRootName() + "." + debugClassName);
	}

	/**
	 *
	 * @param c
	 *            Class
	 * @return Logger
	 */
	@Override
	public de.bsvrz.dua.bastband.bastband.funclib.logger.Logger getLogger(final Class<?> c) {
		return de.bsvrz.dua.bastband.bastband.funclib.logger.StandardLogger
				.getStandardLogger(getRootName() + "." + c.getName());
	}

	@Override
	public de.bsvrz.dua.bastband.bastband.funclib.logger.Logger getLogger(final String s) {
		return de.bsvrz.dua.bastband.bastband.funclib.logger.StandardLogger.getStandardLogger(getRootName() + "." + s);
	}

	@Override
	public void setRootLevel(final de.bsvrz.dua.bastband.bastband.funclib.logger.Logger.Level level) {
		System.err.println("Setting de.logosib.funclib.logger.StandardLogger.JLevel[level.ordinal()]="
				+ de.bsvrz.dua.bastband.bastband.funclib.logger.StandardLogger.JLEVEL[level.ordinal()]);
		java.util.logging.Logger.getLogger(getRootName())
		.setLevel(de.bsvrz.dua.bastband.bastband.funclib.logger.StandardLogger.JLEVEL[level.ordinal()]);
	}

	@Override
	public Object getRootLogger() {
		return java.util.logging.Logger.getLogger(getRootName());
	}

}
