/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.bsvrz.dua.bastband.bastband.dvs.funclib.objectwrapper;

/**
 * History:
 *
 * $Log: AxlLastMessStelle.java,v $
 * Revision 1.2  2007/08/22 07:48:43  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1  2007/07/04 11:43:34  Trier
 * Neuordnung
 *
 * Revision 1.1.1.1  2007/06/20 09:56:26  Trier
 * dvs alles
 *
 * Revision 1.6  2006/04/28 05:48:49  Trier
 * Generics 1.5 verwendet
 *
 * Revision 1.5  2005/11/04 12:13:17  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.4  2005/09/16 08:18:14  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.3  2005/09/09 08:29:05  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.2  2005/09/09 08:10:58  Schawaller
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1  2005/09/09 07:37:27  Trier
 * <Kein Kommentar eingegeben>
 *
 *
 */
import java.util.HashSet;
import java.util.List;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.DataModellMismatchException;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.LogosClientDavConnection;

/**
 * bietet Zugriff auf die {@link LangZeitZaehlStelle LangZeitZaehlStellen}, die
 * auch Achslasten erfassen. Durch die Methode {@link #getAll} werden die
 * DatenModelObjekte vom typ.achsLastMessStelle gelesen und die
 * {@link DEKanal.Axl} der jeweiligen {@link Fahrstreifen} gesetzt.
 */
public class AxlLastMessStelle {

	private static LangZeitZaehlStelle[] __LZZArray;

	public static boolean isNotInitialized() {
		return (AxlLastMessStelle.__LZZArray == null);
	}

	/**
	 * gibt die DatenModellObjekte vom typ langZeitZ&auml;hlstelle zurück, die
	 * auch Achslasten erfassen. DataModellMismatchException wird geworfen, wenn
	 * Objektreferenzen auf falsche Typen verweisen bzw. diese nicht gesetzt
	 * sind.
	 *
	 * @throws DataModellMismatchException
	 *             Instanz konnte nicht ermittelt werden
	 * @return LangZeitZaehlStelle[]
	 */
	public static synchronized LangZeitZaehlStelle[] getAll() throws DataModellMismatchException {

		final HashSet<LangZeitZaehlStelle> lzzSet = new HashSet<>();

		if (AxlLastMessStelle.__LZZArray == null) {
			final LangZeitZaehlStelle[] lzzArray = LangZeitZaehlStelle.getAll();

			final DataModel model = LogosClientDavConnection.getInstance().getDataModel();

			final SystemObjectType typeDeAxl = model.getType("typ.deAxl");
			final SystemObjectType typeFahrstreifen = model.getType("typ.fahrStreifen");

			final AttributeGroup atgAxlLastMessStelle = model.getAttributeGroup("atg.achsLastMessStelle");
			final SystemObjectType soType = model.getType("typ.achsLastMessStelle");
			List<SystemObject> list = null;
			if (soType != null) {
				list = soType.getElements();
			}

			if ((list != null) && (list.size() > 0)) {
				final Data[] dataArray = model.getConfigurationData(list, atgAxlLastMessStelle);

				for (int i = 0; i < list.size(); i++) {

					final ConfigurationObject confObject = (ConfigurationObject) list.get(i);
					final Data oData = dataArray[i];

					final SystemObject soAxlQuelle = oData.getReferenceValue("AchsLastMessStellenQuelle")
							.getSystemObject();
					// System.err.println(soAxlQuelle.getType() + "soAxlQuelle="
					// + soAxlQuelle);
					DEKanal.Axl deKanal = null;
					if ((soAxlQuelle != null) && (soAxlQuelle.isOfType(typeDeAxl))) {
						deKanal = (DEKanal.Axl) DEKanal.getInstance((ConfigurationObject) soAxlQuelle);
					} else {
						throw new DataModellMismatchException("Quelle der achsLastMessStelle " + confObject.getPid()
								+ " ist nicht gesetzt oder nicht vom typ deAxl");
					}

					final SystemObject soFS = oData.getReferenceValue("FahrStreifen").getSystemObject();
					// System.err.println(soFS.getType() + "soFS=" + soFS);
					Fahrstreifen fs = null;
					if ((soFS != null) && (soFS.isOfType(typeFahrstreifen))) {
						fs = Fahrstreifen.getInstance((ConfigurationObject) soFS);
					} else {
						throw new DataModellMismatchException("Fahrstreifen der achsLastMessStelle "
								+ confObject.getPid() + " nicht gesetzt oder nicht vom typ fahrStreifen");
					}
					fs.setDEKanalAxl(deKanal);
					final LangZeitZaehlStelle lzz = fs.getMessQuerschnitt().getAuswerteQuerschnitt()
							.getLangZeitZaehlStelle();

					lzzSet.add(lzz);
				}
			}
			AxlLastMessStelle.__LZZArray = lzzSet.toArray(new LangZeitZaehlStelle[lzzSet.size()]);
		}
		return AxlLastMessStelle.__LZZArray;
	}

}
