/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.bsvrz.dua.bastband.bastband.dvs.funclib.objectwrapper;

/**
 * History:
 *
 * $Log: LangZeitZaehlStelle.java,v $
 * Revision 1.3  2007/08/22 09:24:58  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.2  2007/08/22 07:48:43  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1  2007/07/04 11:44:02  Trier
 * Neuordnung
 *
 * Revision 1.1.1.1  2007/06/20 09:56:26  Trier
 * dvs alles
 *
 * Revision 1.10  2006/11/13 09:24:12  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.9  2006/04/28 05:48:49  Trier
 * Generics 1.5 verwendet
 *
 * Revision 1.8  2005/09/16 08:18:14  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.7  2005/09/08 11:02:38  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.6  2005/09/06 13:59:17  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.5  2005/04/21 09:15:34  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.4  2005/02/28 15:04:15  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.3  2005/02/18 10:08:48  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.2  2005/01/21 13:26:16  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1.1.1  2004/03/23 13:14:19  Trier
 * Basis für LogosApps
 *
 * Revision 1.1.1.1  2004/03/23 07:59:52  Trier
 * vrz
 *
 */
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.ConfigurationObjectType;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.DataModellMismatchException;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.DataModellObjectNotFoundException;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.LogosClientDavConnection;

/**
 * kapselt den DatenModellTyp langZeitZählstelle
 */
public class LangZeitZaehlStelle extends ConfigurationObjectWrapper {

	private static final de.bsvrz.dua.bastband.bastband.funclib.logger.Logger LOGGER = de.bsvrz.dua.bastband.bastband.funclib.logger.LoggerFactory
			.getLoggerFactory().getLogger();

	private static HashMap<String, LangZeitZaehlStelle> __Cache = new HashMap<>();
	private static HashMap<Integer, LangZeitZaehlStelle> __CacheByNr = new HashMap<>();

	private static LangZeitZaehlStelle[] __LZZArray;

	private static ConfigurationObjectType __ConfigurationObjectType;

	private int lzzNummer;
	private String lzzName;
	private Bundesland bundesland;
	private StrassenKlasse strassenKlasse;
	private String lzzArt;
	private String pidStrasse;
	private int vonNK;
	private int nachNK;
	private double station;
	private long beginn;
	private long ende;
	private String klassifizierungsgruppeBAST;

	private int inNK;
	private String inNahziel;
	private String inFernziel;
	private String inStr;
	private AuswerteQuerschnitt inAuswertequerschnitt;

	private int _GegenNK;
	private String _GegenNahziel;
	private String _GegenFernziel;
	private String _GegenStr;
	private AuswerteQuerschnitt _GegenAuswertequerschnitt;

	private boolean _IsFunkStation;

	private AuswerteQuerschnitt[] _AuswerteQuerschnitt;

	private static AttributeGroup __AtgLangZeitZaehlstelle;

	public double getStation() {
		return station;
	}

	public int getBastNummer() {
		return (lzzNummer % 10000);
	}

	/**
	 * gibt die vierstellige Bastnummer als String zur&uuml;ck.
	 *
	 * @return String
	 */
	public String getBastNummerAsString() {
		final int iBastNummer = lzzNummer % 10000;
		String sBastNummer = String.valueOf(iBastNummer);
		while (sBastNummer.length() < 4) {
			sBastNummer = '0' + sBastNummer;
		}
		return sBastNummer;
	}

	/**
	 * gibt die achtstellige Langzeitz&auml;stellenNummer (TK25 + BASt-Nummer) als
	 * String zur&uuml;ck.
	 *
	 * @return String
	 */
	public String getLzzNummerAsString() {
		String sLzzNummer = String.valueOf(lzzNummer);
		while (sLzzNummer.length() < 8) {
			sLzzNummer = '0' + sLzzNummer;
		}
		return sLzzNummer;
	}

	/**
	 * gibt die vierstellige TK-Blatt-Nr als String zur&uuml;ck.
	 *
	 * @return String
	 */
	public String getTK25AsString() {
		final int iTK25 = lzzNummer / 10000;
		String sTK25 = String.valueOf(iTK25);
		while (sTK25.length() < 4) {
			sTK25 = '0' + sTK25;
		}
		return sTK25;
	}

	public AuswerteQuerschnitt getInAuswertequerschnitt() {
		return inAuswertequerschnitt;
	}

	public StrassenKlasse getStrassenKlasse() {
		return strassenKlasse;
	}

	public int getVonNK() {
		return vonNK;
	}

	public String getKlassifizierungsgruppeBAST() {
		return klassifizierungsgruppeBAST;
	}

	public int getAnzahlFahrzeugklassen() {
		if (klassifizierungsgruppeBAST.startsWith("8")) {
			return 9;
		}
		if (klassifizierungsgruppeBAST.startsWith("5")) {
			return 6;
		}
		if (klassifizierungsgruppeBAST.startsWith("2")) {
			return 2;
		}
		return 2;
	}

	public String getPidStrasse() {
		return pidStrasse;
	}

	public Strasse getStrasse() {
		try {
			return new Strasse(pidStrasse);
		} catch (final DataModellMismatchException ex) {
			ex.printStackTrace();
			return null;
		} catch (final DataModellObjectNotFoundException ex) {
			ex.printStackTrace();
			return null;
		} catch (final Exception ex) {
			ex.printStackTrace();
			return null;
		}
	}

	public int getInNK() {
		return inNK;
	}

	public String getGegenStr() {
		return _GegenStr;
	}

	public long getEnde() {
		return ende;
	}

	public String getGegenNahziel() {
		return _GegenNahziel;
	}

	public int getGegenNK() {
		return _GegenNK;
	}

	public String getInNahziel() {
		return inNahziel;
	}

	public String getInFernziel() {
		return inFernziel;
	}

	public Bundesland getBundesland() {
		return bundesland;
	}

	public String getGegenFernziel() {
		return _GegenFernziel;
	}

	public AuswerteQuerschnitt getGegenAuswertequerschnitt() {
		return _GegenAuswertequerschnitt;
	}

	public int getNachNK() {
		return nachNK;
	}

	public long getBeginn() {
		return beginn;
	}

	public String getInStr() {
		return inStr;
	}

	public String getLzzArt() {
		return lzzArt;
	}

	protected LangZeitZaehlStelle(final String pid)
			throws DataModellObjectNotFoundException, DataModellMismatchException {
		super(pid);
	}

	protected LangZeitZaehlStelle(final ConfigurationObject confObj) throws DataModellMismatchException {
		super(confObj);
	}

	@Override
	protected void initData(final ConfigurationObject confObj) throws DataModellMismatchException {

		final Data oData = confObj.getConfigurationData(LangZeitZaehlStelle.__AtgLangZeitZaehlstelle);

		lzzName = oData.getTextValue("ZählstellenName").getText();
		lzzNummer = oData.getUnscaledValue("ZählstellenNummer").intValue();

		if (oData.getUnscaledValue("LandesNummer").isState()) {
			bundesland = Bundesland.getInstanceByCode(oData.getUnscaledValue("LandesNummer").intValue());
		}

		if (oData.getUnscaledValue("StraßenKlasse").isState()) {
			try {
				strassenKlasse = StrassenKlasse
						.getInstanceByName(oData.getUnscaledValue("StraßenKlasse").getState().getName());
			} catch (final Exception e) {
				LangZeitZaehlStelle.LOGGER.error("Fehler bei der Zuordnung der Strassenklasse", e);
			}
		} else {
			LangZeitZaehlStelle.LOGGER.error("%s StrassenKlasse ist kein Status", this.getPID());
		}

		pidStrasse = oData.getReferenceValue("Straße").getSystemObject().getPid();

		if (oData.getUnscaledValue("vonNK").isNumber()) {
			vonNK = oData.getUnscaledValue("vonNK").intValue();
		}
		if (oData.getUnscaledValue("nachNK").isNumber()) {
			nachNK = oData.getUnscaledValue("nachNK").intValue();
		}
		if (oData.getUnscaledValue("Station").isNumber()) {
			station = oData.getUnscaledValue("Station").doubleValue();
		}
		try {
			beginn = oData.getTimeValue("Beginn").getMillis();
		} catch (final Exception e) {
			e.printStackTrace();
		}
		try {
			ende = oData.getTimeValue("Ende").getMillis();
		} catch (final Exception e) {
			e.printStackTrace();
		}

		if (oData.getUnscaledValue("KlassifizierungsgruppeBAST").isState()) {
			klassifizierungsgruppeBAST = oData.getUnscaledValue("KlassifizierungsgruppeBAST").getState().getName();
		}

		Data atlData = oData.getItem("InRichtung");
		if (atlData.getUnscaledValue("NK").isNumber()) {
			inNK = atlData.getUnscaledValue("NK").intValue();
		}
		inNahziel = atlData.getTextValue("Nahziel").getText();
		inFernziel = atlData.getTextValue("Fernziel").getText();
		inStr = atlData.getTextValue("Bezeichnung").getText();
		inAuswertequerschnitt = new AuswerteQuerschnitt(
				(ConfigurationObject) atlData.getReferenceValue("Auswertequerschnitt").getSystemObject());
		inAuswertequerschnitt.setLangZeitZaehlStelle(this);

		atlData = oData.getItem("GegenRichtung");
		if (atlData.getUnscaledValue("NK").isNumber()) {
			_GegenNK = atlData.getUnscaledValue("NK").intValue();
		}
		_GegenNahziel = atlData.getTextValue("Nahziel").getText();
		_GegenFernziel = atlData.getTextValue("Fernziel").getText();
		_GegenStr = atlData.getTextValue("Bezeichnung").getText();
		_GegenAuswertequerschnitt = new AuswerteQuerschnitt(
				(ConfigurationObject) atlData.getReferenceValue("Auswertequerschnitt").getSystemObject());
		_GegenAuswertequerschnitt.setLangZeitZaehlStelle(this);
		final List<SystemObject> listMQ = confObj.getObjectSet("AuswerteQuerschnitte").getElements();
		_AuswerteQuerschnitt = new AuswerteQuerschnitt[listMQ.size()];
		for (int mqNr = 0; (mqNr < listMQ.size()); mqNr++) {
			_AuswerteQuerschnitt[mqNr] = new AuswerteQuerschnitt((ConfigurationObject) listMQ.get(mqNr));
		}

		// bestimmen ob an Funk oder nicht
		if (inAuswertequerschnitt != null) {
			if (inAuswertequerschnitt.getMessQuerschnitt() != null) {
				final Fahrstreifen[] fsArray = inAuswertequerschnitt.getMessQuerschnitt().getFahrstreifen();
				if ((fsArray != null) && (fsArray.length > 0) && (fsArray[0] != null)) {
					final FahrstreifenQuelle quelle = fsArray[0].getFahrstreifenQuelle();
					if ((quelle != null) && (quelle instanceof DEKanal.Lve)) {
						final EAK eak = ((DEKanal.Lve) quelle).getEAK();
						if (eak != null) {
							final SteuerModul sm = eak.getSteuerModul();
							if (sm != null) {
								final KriBus bus = sm.getBus();
								if (bus != null) {
									_IsFunkStation = bus.isFunkBus();
								} else {
									LangZeitZaehlStelle.LOGGER.info("kein Bus für SteuerModul %s", sm.getPID());
								}
							} else {
								LangZeitZaehlStelle.LOGGER.info("kein SteuerModul für EAK %s", eak.getPID());
							}
						} else {
							LangZeitZaehlStelle.LOGGER.debug("kein EAK für Quelle %s", quelle);
						}
					}
				}
			}

		}

		LangZeitZaehlStelle.__Cache.put(this.getPID(), this);
		// System.out.println("LZZ __CacheByNr.put: " + this.getBastNummer());
		LangZeitZaehlStelle.__CacheByNr.put(this.getBastNummer(), this);
	}

	/**
	 * gibt die Nummer der Zählstelle zuück
	 *
	 * @return int
	 */
	public int getLzzNummer() {
		return lzzNummer;
	}

	/**
	 * gibt den Namen der Zählstelle zuück
	 *
	 * @return int
	 */
	public String getLzzName() {
		return lzzName;
	}

	public boolean isFunkstation() {
		return _IsFunkStation;
	}

	/**
	 * gibt die zugehörigen MessQuerschnitte zurück
	 *
	 * @return MessQuerschnitt[]
	 */
	public AuswerteQuerschnitt[] getAuswerteQuerschnitte() {
		return _AuswerteQuerschnitt;
	}

	/**
	 * gibt die Instance der LangZeitZaehlStelle mit der PID zurück. Es wird eine
	 * DataModellObjectNotFoundException geworfen, wenn die PID nicht vorhanden ist
	 * oder das zur PID gehörende Objekt nich vom typ langZeitZählstelle ist. Eine
	 * DataModellMismatchException wird geworfen, wenn Objektreferenzen auf falsche
	 * Typen verweisen bzw. diese nicht gesetzt sind.
	 *
	 * @param pid
	 *            String
	 * @throws DataModellObjectNotFoundException
	 *             ungültiges Datenmodell
	 * @throws DataModellMismatchException
	 *             ungültiges Datenmodell
	 * @return LangZeitZaehlStelle
	 */
	public static LangZeitZaehlStelle getInstance(final String pid)
			throws DataModellObjectNotFoundException, DataModellMismatchException {

		LangZeitZaehlStelle retValue = LangZeitZaehlStelle.__Cache.get(pid);
		if (retValue == null) {
			retValue = new LangZeitZaehlStelle(pid);
		}
		return retValue;
	}

	public static LangZeitZaehlStelle getInstance(final int bastNr) throws DataModellMismatchException {

		LangZeitZaehlStelle retValue = LangZeitZaehlStelle.__CacheByNr.get(bastNr);
		if (retValue == null) {
			LangZeitZaehlStelle.getAll();
		}
		retValue = LangZeitZaehlStelle.__CacheByNr.get(bastNr);
		return retValue;
	}

	public static void sortByName(final LangZeitZaehlStelle[] lzzArray) {
		final Comparator<LangZeitZaehlStelle> comparator = new Comparator<LangZeitZaehlStelle>() {
			@Override
			public boolean equals(final Object o) {
				return (o == this);
			}

			@Override
			public int compare(final LangZeitZaehlStelle lzz01, final LangZeitZaehlStelle lzz02) {
				final String s1 = lzz01.getName();
				final String s2 = lzz02.getName();
				if ((s1 != null) && (s2 != null)) {
					return (s1.compareTo(s2));
				}
				if (s1 == null) {
					return 1;
				}
				return -1;
			}
		};

		Arrays.sort(lzzArray, comparator);

	}

	/**
	 * gibt die DatenModellObjekte vom typ langZeitZählstelle zurück, wenn keiner
	 * dieser Objekte Fehler in den ObjektReferenzen aufweist.
	 * DataModellMismatchException wird geworfen, wenn Objektreferenzen auf falsche
	 * Typen verweisen bzw. diese nicht gesetzt sind.
	 *
	 * @throws DataModellMismatchException
	 *             das Datenmodell ist ungültig
	 * @return LangZeitZaehlStelle[]
	 */
	public static synchronized LangZeitZaehlStelle[] getAll() throws DataModellMismatchException {

		if (LangZeitZaehlStelle.__LZZArray == null) {
			final DataModel model = LogosClientDavConnection.getInstance().getDataModel();
			final List<SystemObject> list = model.getType("typ.langZeitZählstelle").getElements();
			LangZeitZaehlStelle.__LZZArray = new LangZeitZaehlStelle[list.size()];
			for (int i = 0; i < list.size(); i++) {
				LangZeitZaehlStelle.__LZZArray[i] = new LangZeitZaehlStelle((ConfigurationObject) list.get(i));
			}
		}
		return LangZeitZaehlStelle.__LZZArray;
	}

	/**
	 * gibt die LangzeitZ&auml;hlstellenWrapperObjekte zur&uuml;ck, die der
	 * Verkehrmengenerfassung dienen und die keine Fehler in den ObjektReferenzen
	 * aufweisen. DataModellMismatchException wird geworfen, wenn Objektreferenzen
	 * auf falsche Typen verweisen bzw. diese nicht gesetzt sind.
	 *
	 * @throws DataModellMismatchException
	 *             das Datenmodell ist ungültig
	 * @return LangZeitZaehlStelle[]
	 */
	public static LangZeitZaehlStelle[] getAllLve() throws DataModellMismatchException {

		final LangZeitZaehlStelle[] lzzArray = LangZeitZaehlStelle.getAll();

		final ArrayList<LangZeitZaehlStelle> list = new ArrayList<>(lzzArray.length);

		for (int i = 0; i < lzzArray.length; i++) {
			if (!lzzArray[i].isAchslastErfassung()) {
				list.add(lzzArray[i]);
			}
		}
		return list.toArray(new LangZeitZaehlStelle[list.size()]);
	}

	/**
	 * gibt die LangzeitZ&auml;hlstellenWrapperObjekte zur&uuml;ck, die der
	 * Achlastdatenfassung dienen und die keine Fehler in den ObjektReferenzen
	 * aufweisen. DataModellMismatchException wird geworfen, wenn Objektreferenzen
	 * auf falsche Typen verweisen bzw. diese nicht gesetzt sind.
	 *
	 * @throws DataModellMismatchException
	 *             das Datenmodell ist ungültig
	 * @return LangZeitZaehlStelle[]
	 */
	public static LangZeitZaehlStelle[] getAllAxl() throws DataModellMismatchException {

		final LangZeitZaehlStelle[] lzzArray = LangZeitZaehlStelle.getAll();

		final ArrayList<LangZeitZaehlStelle> list = new ArrayList<>(2);

		for (final LangZeitZaehlStelle element : lzzArray) {
			if (element.isAchslastErfassung()) {
				list.add(element);
			}
		}
		return list.toArray(new LangZeitZaehlStelle[list.size()]);
	}

	@Override
	public String toString() {
		final StringBuffer sb = new StringBuffer();
		sb.append(lzzNummer).append(" - ").append(lzzName);
		return sb.toString();
	}

	public String getMultilineRepresentation(final int indent) {
		final String lf = System.getProperty("line.separator");
		final StringBuffer sIndent = new StringBuffer();
		for (int i = 0; i < indent; i++) {
			sIndent.append(' ');
		}
		final StringBuffer sb = new StringBuffer();

		sb.append(sIndent).append("PID: \"").append(getPID()).append("\"").append(lf);
		sb.append(sIndent).append("Name: \"").append(getName()).append("\"").append(lf);

		sb.append(sIndent).append("ZählstellenNummer: \"").append(lzzNummer).append("\"").append(lf);
		sb.append(sIndent).append("ZählstellenName: \"").append(lzzName).append("\"").append(lf);

		sb.append(sIndent).append("Land: \"").append(bundesland).append("\"").append(lf);
		sb.append(sIndent).append("Strassenklasse: \"").append(strassenKlasse).append("\"").append(lf);
		sb.append(sIndent).append("Zählstellenart: \"").append(lzzArt).append("\"").append(lf);
		sb.append(sIndent).append("Strasse: \"").append(pidStrasse).append("\"").append(lf);
		sb.append(sIndent).append("VonNK: \"").append(vonNK).append("\"").append(lf);
		sb.append(sIndent).append("NachNK: \"").append(nachNK).append("\"").append(lf);
		sb.append(sIndent).append("Station: \"").append(station).append("\"").append(lf);
		if (beginn == 0) {
			sb.append(sIndent).append("Beginn: \"nicht bekannt\"").append(lf);
		} else {
			sb.append(sIndent).append("Beginn: \"").append(new Date(beginn)).append("\"").append(lf);
		}
		if (ende == 0) {
			sb.append(sIndent).append("Ende: \"-\"").append(lf);
		} else {
			sb.append(sIndent).append("Ende: \"").append(new Date(ende)).append("\"").append(lf);
		}
		sb.append(sIndent).append("KlassifizierungsgruppeBAST: \"").append(klassifizierungsgruppeBAST).append("\"")
				.append(lf);

		sb.append(sIndent).append("InRichtung:").append(lf);
		sb.append(sIndent).append("  NK: \"").append(inNK).append("\"").append(lf);
		sb.append(sIndent).append("  Nahziel: \"").append(inNahziel).append("\"").append(lf);
		sb.append(sIndent).append("  Fernziel: \"").append(inFernziel).append("\"").append(lf);
		sb.append(sIndent).append("  Str: \"").append(inStr).append("\"").append(lf);
		sb.append(sIndent).append("  Auswertequerschnitt:").append(lf);
		sb.append(inAuswertequerschnitt.getMultilineRepresentation(indent + 8));

		sb.append(sIndent).append("GegenRichtung:").append(lf);
		sb.append(sIndent).append("  NK: \"").append(_GegenNK).append("\"").append(lf);
		sb.append(sIndent).append("  Nahziel: \"").append(_GegenNahziel).append("\"").append(lf);
		sb.append(sIndent).append("  Fernziel: \"").append(_GegenFernziel).append("\"").append(lf);
		sb.append(sIndent).append("  Str: \"").append(_GegenStr).append("\"").append(lf);
		sb.append(sIndent).append("  Auswertequerschnitt:").append(lf);
		sb.append(_GegenAuswertequerschnitt.getMultilineRepresentation(indent + 8));

		sb.append(sIndent).append("Messquerschnitte:").append(lf);
		if ((_AuswerteQuerschnitt != null) && (_AuswerteQuerschnitt.length > 0)) {
			for (int i = 0; i < _AuswerteQuerschnitt.length; i++) {
				sb.append(sIndent).append("  Messquerschnitt " + i + ": "
						+ _AuswerteQuerschnitt[i].getMultilineRepresentation(indent + 8));
			}
		} else {
			sb.append(sIndent).append("  keine Messquerschnitte vorhanden");
		}
		return sb.toString();
	}

	/**
	 * getMultilineRepresentation
	 *
	 * @return String
	 */
	public String getMultilineRepresentation() {
		return getMultilineRepresentation(0);
	}

	@Override
	public String getTreeString() {
		return this.getName();
	}

	@Override
	public String getToolTipText() {
		try {
			return this.getStrasse().getTreeString() + ' ' + this.getInFernziel() + " - " + this.getGegenFernziel();
		} catch (final Exception e) {
			return null;
		}
	}

	@Override
	ConfigurationObjectType getType() {
		if (LangZeitZaehlStelle.__ConfigurationObjectType == null) {
			final DataModel model = LogosClientDavConnection.getInstance().getDataModel();
			LangZeitZaehlStelle.__ConfigurationObjectType = (ConfigurationObjectType) model
					.getType("typ.langZeitZählstelle");
			LangZeitZaehlStelle.__AtgLangZeitZaehlstelle = model.getAttributeGroup("atg.langZeitZählstelle");
			final List<SystemObject> list = LangZeitZaehlStelle.__ConfigurationObjectType.getObjects();
			model.getConfigurationData(list, LangZeitZaehlStelle.__AtgLangZeitZaehlstelle);
		}
		return LangZeitZaehlStelle.__ConfigurationObjectType;
	}

	/**
	 * gibt die Fahrstreifen der Z&auml;hlstelle in der Reihenfolge InRichtg.HFS,
	 * InRichtg.&Uuml;FS01, InRichtg.&Uuml;FS02 .... GegenRichtg.&Uuml;FS02,
	 * GegenRichtg.&Uuml;FS01, GegenRichtg.HFS zur&uuml;ck.
	 *
	 * @return ArrayList
	 */
	public Fahrstreifen[] getFahrstreifen() {
		Fahrstreifen[] fsInArray = null;
		Fahrstreifen[] fsGegenArray = null;
		final AuswerteQuerschnitt aq = this.inAuswertequerschnitt;
		if (aq != null) {
			final MessQuerschnitt mq = aq.getMessQuerschnitt();
			if (mq != null) {
				fsInArray = mq.getFahrstreifen();
			}
		}
		final AuswerteQuerschnitt gegenAuswertequerschnitt = _GegenAuswertequerschnitt;
		if (gegenAuswertequerschnitt != null) {
			final MessQuerschnitt mq = gegenAuswertequerschnitt.getMessQuerschnitt();
			if (mq != null) {
				fsGegenArray = mq.getFahrstreifen();
			}
		}

		// die Fahrstreifen werden von rechts (HauptFS) nach links (ÜFS01,
		// ÜFS02..) zurückgegeben.
		// die Ausgabe in das BASt-Format erfolgt aber in der Reihenfolge
		// HFS 1.RFB, ÜFS01 1.RFB, ÜFS02 1.RFB, ... , ÜFS02 2.RFB, ÜFS01 2.RFB,
		// HFS 2.RFB
		final ArrayList<Fahrstreifen> fsList = new ArrayList<>();
		if (fsInArray != null) {
			for (final Fahrstreifen element : fsInArray) {
				fsList.add(element);
			}
		}
		if (fsGegenArray != null) {
			for (int i = (fsGegenArray.length - 1); i >= 0; i--) {
				fsList.add(fsGegenArray[i]);
			}
		}
		return fsList.toArray(new Fahrstreifen[fsList.size()]);
	}

	/**
	 * gibt true zur&uuml;ck, wenn die Zählstelle Achslastdaten erfasst.
	 *
	 * @return boolean
	 * @throws DataModellMismatchException
	 *             das Datenmodell ist ungültig
	 */
	public boolean isAchslastErfassung() throws DataModellMismatchException {
		boolean bReturnValue = false;
		final Fahrstreifen[] fsArray = getFahrstreifen();
		for (int i = 0; ((i < fsArray.length) && (!bReturnValue)); i++) {
			bReturnValue = fsArray[i].isAchslastErfassung();
		}
		return bReturnValue;
	}

}
